@@ -31,6 +31,7 @@ import (
 
 type settingsSource interface {
 	GetAppInstanceLabelKey() (string, error)
+	GetTrackingMethod() (string, error)
 }
 
 var _ settingsSource = &settings.SettingsManager{}
@@ -153,13 +154,15 @@ func affectedRevisionInfo(payloadIf interface{}) (webURLs []string, revision str
 	case bitbucketserver.RepositoryReferenceChangedPayload:
 
 		// Webhook module does not parse the inner links
-		for _, l := range payload.Repository.Links["clone"].([]interface{}) {
-			link := l.(map[string]interface{})
-			if link["name"] == "http" {
-				webURLs = append(webURLs, link["href"].(string))
-			}
-			if link["name"] == "ssh" {
-				webURLs = append(webURLs, link["href"].(string))
+		if payload.Repository.Links != nil {
+			for _, l := range payload.Repository.Links["clone"].([]interface{}) {
+				link := l.(map[string]interface{})
+				if link["name"] == "http" {
+					webURLs = append(webURLs, link["href"].(string))
+				}
+				if link["name"] == "ssh" {
+					webURLs = append(webURLs, link["href"].(string))
+				}
 			}
 		}
 
@@ -214,6 +217,11 @@ func (a *ArgoCDWebhookHandler) HandleEvent(payload interface{}) {
 		return
 	}
 
+	trackingMethod, err := a.settingsSrc.GetTrackingMethod()
+	if err != nil {
+		log.Warnf("Failed to get trackingMethod: %v", err)
+		return
+	}
 	appInstanceLabelKey, err := a.settingsSrc.GetAppInstanceLabelKey()
 	if err != nil {
 		log.Warnf("Failed to get appInstanceLabelKey: %v", err)
@@ -221,19 +229,11 @@ func (a *ArgoCDWebhookHandler) HandleEvent(payload interface{}) {
 	}
 
 	for _, webURL := range webURLs {
-		urlObj, err := url.Parse(webURL)
+		repoRegexp, err := getWebUrlRegex(webURL)
 		if err != nil {
-			log.Warnf("Failed to parse repoURL '%s'", webURL)
+			log.Warnf("Failed to get repoRegexp: %s", err)
 			continue
 		}
-
-		regexpStr := `(?i)(http://|https://|\w+@|ssh://(\w+@)?)` + urlObj.Hostname() + "(:[0-9]+|)[:/]" + urlObj.Path[1:] + "(\\.git)?"
-		repoRegexp, err := regexp.Compile(regexpStr)
-		if err != nil {
-			log.Warnf("Failed to compile regexp for repoURL '%s'", webURL)
-			continue
-		}
-
 		for _, app := range apps.Items {
 			if appRevisionHasChanged(&app, revision, touchedHead) && appUsesURL(&app, webURL, repoRegexp) {
 				if appFilesHaveChanged(&app, changedFiles) {
@@ -243,7 +243,7 @@ func (a *ArgoCDWebhookHandler) HandleEvent(payload interface{}) {
 						continue
 					}
 				} else if change.shaBefore != "" && change.shaAfter != "" {
-					if err := a.storePreviouslyCachedManifests(&app, change, appInstanceLabelKey); err != nil {
+					if err := a.storePreviouslyCachedManifests(&app, change, trackingMethod, appInstanceLabelKey); err != nil {
 						log.Warnf("Failed to store cached manifests of previous revision for app '%s': %v", app.Name, err)
 					}
 				}
@@ -252,7 +252,26 @@ func (a *ArgoCDWebhookHandler) HandleEvent(payload interface{}) {
 	}
 }
 
-func (a *ArgoCDWebhookHandler) storePreviouslyCachedManifests(app *v1alpha1.Application, change changeInfo, appInstanceLabelKey string) error {
+// getWebUrlRegex compiles a regex that will match any targetRevision referring to the same repo as the given webURL.
+// webURL is expected to be a URL from an SCM webhook payload pointing to the web page for the repo.
+func getWebUrlRegex(webURL string) (*regexp.Regexp, error) {
+	urlObj, err := url.Parse(webURL)
+	if err != nil {
+		return nil, fmt.Errorf("failed to parse repoURL '%s'", webURL)
+	}
+
+	regexEscapedHostname := regexp.QuoteMeta(urlObj.Hostname())
+	regexEscapedPath := regexp.QuoteMeta(urlObj.Path[1:])
+	regexpStr := fmt.Sprintf(`(?i)^(http://|https://|\w+@|ssh://(\w+@)?)%s(:[0-9]+|)[:/]%s(\.git)?$`, regexEscapedHostname, regexEscapedPath)
+	repoRegexp, err := regexp.Compile(regexpStr)
+	if err != nil {
+		return nil, fmt.Errorf("failed to compile regexp for repoURL '%s'", webURL)
+	}
+
+	return repoRegexp, nil
+}
+
+func (a *ArgoCDWebhookHandler) storePreviouslyCachedManifests(app *v1alpha1.Application, change changeInfo, trackingMethod string, appInstanceLabelKey string) error {
 	err := argo.ValidateDestination(context.Background(), &app.Spec.Destination, a.db)
 	if err != nil {
 		return err
@@ -264,10 +283,10 @@ func (a *ArgoCDWebhookHandler) storePreviouslyCachedManifests(app *v1alpha1.Appl
 		return err
 	}
 	var cachedManifests cache.CachedManifestResponse
-	if err := a.repoCache.GetManifests(change.shaBefore, &app.Spec.Source, &clusterInfo, app.Spec.Destination.Namespace, appInstanceLabelKey, app.Name, &cachedManifests); err == nil {
+	if err := a.repoCache.GetManifests(change.shaBefore, &app.Spec.Source, &clusterInfo, app.Spec.Destination.Namespace, trackingMethod, appInstanceLabelKey, app.Name, &cachedManifests); err == nil {
 		return err
 	}
-	if err = a.repoCache.SetManifests(change.shaAfter, &app.Spec.Source, &clusterInfo, app.Spec.Destination.Namespace, appInstanceLabelKey, app.Name, &cachedManifests); err != nil {
+	if err = a.repoCache.SetManifests(change.shaAfter, &app.Spec.Source, &clusterInfo, app.Spec.Destination.Namespace, trackingMethod, appInstanceLabelKey, app.Name, &cachedManifests); err != nil {
 		return err
 	}
 	return nil
@@ -336,8 +355,14 @@ func appRevisionHasChanged(app *v1alpha1.Application, revision string, touchedHe
 	if targetRev == "HEAD" || targetRev == "" { // revision is head
 		return touchedHead
 	}
+	targetRevisionHasPrefixList := []string{"refs/heads/", "refs/tags/"}
+	for _, prefix := range targetRevisionHasPrefixList {
+		if strings.HasPrefix(app.Spec.Source.TargetRevision, prefix) {
+			return revision == targetRev
+		}
+	}
 
-	return targetRev == revision
+	return app.Spec.Source.TargetRevision == revision
 }
 
 func appUsesURL(app *v1alpha1.Application, webURL string, repoRegexp *regexp.Regexp) bool {
